@@ -5,12 +5,16 @@ from scipy.signal import medfilt2d, find_peaks
 from skimage.measure import label
 from skimage.morphology import closing, disk
 
+from math import floor
+
 import multiprocessing as mp
 from itertools import repeat as irepeat
 
 from bmlab import Session
 from bmlab.fits import fit_vipa, VIPA, fit_lorentz_region
 from bmlab.image import extract_lines_along_arc, find_max_in_radius
+from bmlab.export import FluorescenceExport,\
+    FluorescenceCombinedExport, BrillouinExport
 
 import warnings
 
@@ -57,7 +61,12 @@ class ExtractionController(object):
                 calib_key, time, new_point[0], new_point[1])
 
     def find_points_all(self):
-        for calib_key in self.session.get_calib_keys():
+        calib_keys = self.session.get_calib_keys()
+
+        if not calib_keys:
+            return
+
+        for calib_key in calib_keys:
             self.find_points(calib_key)
 
     def find_points(self, calib_key, min_height=10,
@@ -72,6 +81,14 @@ class ExtractionController(object):
         if time is None:
             return
 
+        # Account for binning for default values
+        disc_size = 10
+        binning = self.session.get_calibration_binning_factor(calib_key)
+        if binning:
+            max_distance = max_distance / binning
+            disc_size = disc_size / binning
+            min_area = min_area / (binning**2)
+
         img = medfilt2d(img)
         # This is the background level
         threshold = np.median(img)
@@ -83,7 +100,7 @@ class ExtractionController(object):
         img_peaks = img[img > threshold + min_height]
         # Calculate the signal dependent peak threshold
         height = (np.nanmean(img_peaks) + threshold) / 2
-        img_closed = closing(img > height, disk(10))
+        img_closed = closing(img > height, disk(disc_size))
 
         # Find all peaks higher than the min_height
         image_label, num = label(img_closed, return_num=True)
@@ -112,12 +129,7 @@ class ExtractionController(object):
             all_peaks)
 
         # Add found peaks to model
-        em.clear_points(calib_key)
-
-        for p in peaks:
-            # Warning: x-axis in imshow is 1-axis in img, y-axis is 0-axis
-            em.add_point(
-                calib_key, time, p[0], p[1])
+        em.set_points(calib_key, time, list(peaks))
 
     def distance_point_to_line(self, point, line0, line1):
         return abs(
@@ -126,15 +138,63 @@ class ExtractionController(object):
                / np.sqrt((line1[1] - line0[1])**2 + (line1[0] - line0[0])**2)
 
 
-class CalibrationController(object):
+class ImageController(object):
 
-    def __init__(self):
+    def __init__(self, model, get_image, get_time, get_exposure):
         self.session = Session.get_instance()
+        self.model = model
+        self.get_image = get_image
+        self.get_time = get_time
+        self.get_exposure = get_exposure
+
+    def extract_spectra(self, image_key, frame_num=None):
+        em = self.session.extraction_model()
+        if not em:
+            return None, None, None
+        time = self.get_time(image_key)
+        arc = em.get_arc_by_time(time)
+        if arc.size == 0:
+            return None, None, None
+
+        imgs = self.get_image(image_key)
+        if frame_num is not None:
+            imgs = imgs[frame_num:frame_num+1]
+
+        # Extract values from *all* frames in the current calibration
+        spectra = []
+        for img in imgs:
+            values_by_img = extract_lines_along_arc(
+                img,
+                arc
+            )
+            spectra.append(values_by_img)
+
+        exposure = self.get_exposure(image_key)
+        times = exposure * np.arange(len(imgs)) + time
+
+        intensities = np.nanmean(imgs, axis=(1, 2))
+
+        # We only set the spectra if we extracted all
+        if frame_num is None:
+            self.model().set_spectra(image_key, spectra)
+        return spectra, times, intensities
+
+
+class CalibrationController(ImageController):
+
+    def __init__(self, *args, **kwargs):
+        session = Session.get_instance()
+        super(CalibrationController, self).__init__(
+            model=session.calibration_model,
+            get_image=session.get_calibration_image,
+            get_time=session.get_calibration_time,
+            get_exposure=session.get_calibration_exposure
+        )
         return
 
     def find_peaks(self, calib_key, min_prominence=15,
                    num_brillouin_samples=2, min_height=15):
-        spectra = self.extract_calibration_spectra(calib_key)
+        spectra, _, _ = self.extract_spectra(calib_key)
         if spectra is None:
             return
         spectrum = np.mean(spectra, axis=0)
@@ -247,7 +307,7 @@ class CalibrationController(object):
                 max_count.value = -1
             return
 
-        spectra = self.extract_calibration_spectra(calib_key)
+        spectra, _, _ = self.extract_spectra(calib_key)
         time = self.session.get_calibration_time(calib_key)
 
         if spectra is None or len(spectra) == 0:
@@ -330,32 +390,6 @@ class CalibrationController(object):
                 cm.add_brillouin_fit(calib_key, region_key, frame_num,
                                      w0s, fwhms, intensities, offset)
 
-    def extract_calibration_spectra(self, calib_key, frame_num=None):
-        em = self.session.extraction_model()
-        cm = self.session.calibration_model()
-        if not em:
-            return
-        arc = em.get_arc_by_calib_key(calib_key)
-        if arc.size == 0:
-            return
-
-        imgs = self.session.get_calibration_image(calib_key)
-        if frame_num is not None:
-            imgs = imgs[frame_num:frame_num+1]
-
-        # Extract values from *all* frames in the current calibration
-        spectra = []
-        for img in imgs:
-            values_by_img = extract_lines_along_arc(
-                img,
-                arc
-            )
-            spectra.append(values_by_img)
-        # We only set the spectra if we extracted all
-        if frame_num is None:
-            cm.set_spectra(calib_key, spectra)
-        return spectra
-
     def expected_frequencies(self, calib_key=None, current_frame=None):
         cm = self.session.calibration_model()
 
@@ -395,10 +429,16 @@ class PeakSelectionController(object):
         psm.add_rayleigh_region(region_pix)
 
 
-class EvaluationController(object):
+class EvaluationController(ImageController):
 
-    def __init__(self):
-        self.session = Session.get_instance()
+    def __init__(self, *args, **kwargs):
+        session = Session.get_instance()
+        super(EvaluationController, self).__init__(
+            model=session.evaluation_model,
+            get_image=session.get_payload_image,
+            get_time=session.get_payload_time,
+            get_exposure=session.get_payload_exposure
+        )
         return
 
     def evaluate(self, abort=None, count=None, max_count=None):
@@ -426,10 +466,10 @@ class EvaluationController(object):
                 max_count.value = -1
             return
 
-        image_keys = self.session.get_image_keys()
+        image_keys = self.session.get_image_keys(True)
 
         if max_count is not None:
-            max_count.value += len(image_keys)
+            max_count.value = len(image_keys)
 
         brillouin_regions = pm.get_brillouin_regions()
         rayleigh_regions = pm.get_rayleigh_regions()
@@ -437,7 +477,12 @@ class EvaluationController(object):
         resolution = self.session.get_payload_resolution()
 
         # Get first spectrum to find number of images
-        spectra, _, _ = self.extract_payload_spectra('0')
+        spectra, _, _ = self.extract_spectra('0')
+
+        if not spectra:
+            if max_count is not None:
+                max_count.value = -1
+            return
 
         # We create a variable for this value here,
         # so changing nr_brillouin_peaks during evaluation
@@ -462,120 +507,141 @@ class EvaluationController(object):
 
         pool_size = mp.cpu_count() * 2
         pool = mp.Pool(processes=pool_size)
-
+        # Initialize the Rayleigh shift
+        # used for compensating drifts
+        rayleigh_peak_initial =\
+            np.nan * np.ones((len(rayleigh_regions)))
+        rayleigh_shift = 0
         # Loop over all measurement positions
-        ind_total = 0
-        for ind_x in range(resolution[0]):
-            for ind_y in range(resolution[1]):
-                for ind_z in range(resolution[2]):
-                    # Calculate the image key for the given position
-                    image_key = str(ind_z * (resolution[0] * resolution[1])
-                                    + ind_y * resolution[0] + ind_x)
-                    # Check that this image key actually exists
-                    # (in case of e.g. an aborted measurement it does not)
-                    if image_key not in image_keys:
-                        continue
+        for idx, image_key in enumerate(image_keys):
+            # Calculate the indices for the given key
+            (ind_x, ind_y, ind_z) =\
+                self.get_indices_from_key(resolution, image_key)
 
-                    if (abort is not None) and abort.value:
-                        calculate_derived_values()
-                        if max_count is not None:
-                            max_count.value = -1
-                        return
-                    spectra, times, intensities =\
-                        self.extract_payload_spectra(image_key)
-                    if spectra is None:
-                        continue
-                    evm.results['time'][ind_x, ind_y, ind_z, :, 0, 0] =\
-                        times
-                    evm.results['intensity'][ind_x, ind_y, ind_z, :, 0, 0] =\
-                        intensities
+            if (abort is not None) and abort.value:
+                calculate_derived_values()
+                if max_count is not None:
+                    max_count.value = -1
+                return
+            spectra, times, intensities =\
+                self.extract_spectra(image_key)
+            if spectra is None:
+                continue
+            evm.results['time'][ind_x, ind_y, ind_z, :, 0, 0] =\
+                times
+            evm.results['intensity'][ind_x, ind_y, ind_z, :, 0, 0] =\
+                intensities
 
-                    # Pack the data for parallel processing
-                    regions = brillouin_regions + rayleigh_regions
-                    packed_data = zip(irepeat(spectra), regions)
-                    # Process it
-                    results = pool.starmap(self.fit_spectra, packed_data)
+            # We shift the evaluated regions here
+            # to compensate for eventual drift
+            brillouin_regions_shifted = [
+                tuple([val + rayleigh_shift for val in region])
+                for region in brillouin_regions]
+            rayleigh_regions_shifted = [
+                tuple([val + rayleigh_shift for val in region])
+                for region in rayleigh_regions]
 
-                    # Unpack the results
-                    for frame_num, spectrum in enumerate(spectra):
-                        for region_key, region in enumerate(brillouin_regions):
-                            ind = (ind_x, ind_y, ind_z,
-                                   frame_num, region_key, 0)
-                            evm.results['brillouin_peak_position'][ind] =\
-                                results[region_key][frame_num][0]
-                            evm.results['brillouin_peak_fwhm'][ind] =\
-                                results[region_key][frame_num][1]
-                            evm.results['brillouin_peak_intensity'][ind] =\
-                                results[region_key][frame_num][2]
+            # Pack the data for parallel processing
+            regions = brillouin_regions_shifted + rayleigh_regions_shifted
+            packed_data = zip(irepeat(spectra), regions)
+            # Process it
+            results = pool.starmap(self.fit_spectra, packed_data)
 
-                        for region_key, region \
-                                in enumerate(rayleigh_regions,
-                                             start=len(brillouin_regions)):
-                            ind = (ind_x, ind_y, ind_z, frame_num,
-                                   region_key - len(brillouin_regions))
-                            evm.results['rayleigh_peak_position'][ind] =\
-                                results[region_key][frame_num][0]
-                            evm.results['rayleigh_peak_fwhm'][ind] =\
-                                results[region_key][frame_num][1]
-                            evm.results['rayleigh_peak_intensity'][ind] =\
-                                results[region_key][frame_num][2]
+            # Unpack the results
+            for frame_num, spectrum in enumerate(spectra):
+                for region_key, _ in enumerate(brillouin_regions):
+                    ind = (ind_x, ind_y, ind_z,
+                           frame_num, region_key, 0)
+                    evm.results['brillouin_peak_position'][ind] =\
+                        results[region_key][frame_num][0]
+                    evm.results['brillouin_peak_fwhm'][ind] =\
+                        results[region_key][frame_num][1]
+                    evm.results['brillouin_peak_intensity'][ind] =\
+                        results[region_key][frame_num][2]
 
-                    # We can only do a multi-peak fit after the single-peak
-                    # Rayleigh fit is done, because we have to know the
-                    # Rayleigh peak positions in GHz in order to convert
-                    # the multi-peak fit bounds given in GHz into the position
-                    # in pixels.
-                    if nr_brillouin_peaks > 1:
-                        ind =\
-                            (ind_x, ind_y, ind_z, slice(None), slice(None), 0)
-                        rayleigh_peaks = np.transpose(
-                            evm.results['rayleigh_peak_position'][ind]
-                        )
-                        bounds = self.create_bounds(
-                            brillouin_regions,
-                            times,
-                            rayleigh_peaks
-                        )
-                        if bounds is not None:
-                            packed_data_multi_peak =\
-                                zip(irepeat(spectra), brillouin_regions,
-                                    irepeat(nr_brillouin_peaks),
-                                    bounds)
-                        else:
-                            packed_data_multi_peak =\
-                                zip(irepeat(spectra), brillouin_regions,
-                                    irepeat(nr_brillouin_peaks),
-                                    irepeat(bounds))
-                        # Process it
-                        results_multi_peak = pool.starmap(
-                            self.fit_spectra, packed_data_multi_peak)
+                for region_key, _ \
+                        in enumerate(rayleigh_regions,
+                                     start=len(brillouin_regions)):
+                    ind = (ind_x, ind_y, ind_z, frame_num,
+                           region_key - len(brillouin_regions))
+                    evm.results['rayleigh_peak_position'][ind] =\
+                        results[region_key][frame_num][0]
+                    evm.results['rayleigh_peak_fwhm'][ind] =\
+                        results[region_key][frame_num][1]
+                    evm.results['rayleigh_peak_intensity'][ind] =\
+                        results[region_key][frame_num][2]
 
-                        for frame_num, spectrum in enumerate(spectra):
-                            for region_key, region in enumerate(
-                                    brillouin_regions):
-                                ind = (ind_x, ind_y, ind_z,
-                                       frame_num, region_key,
-                                       slice(1, nr_brillouin_peaks+1))
-                                evm.results[
-                                    'brillouin_peak_position'][ind] = \
-                                    results_multi_peak[
-                                        region_key][frame_num][0]
-                                evm.results[
-                                    'brillouin_peak_fwhm'][ind] = \
-                                    results_multi_peak[
-                                        region_key][frame_num][1]
-                                evm.results[
-                                    'brillouin_peak_intensity'][ind] = \
-                                    results_multi_peak[
-                                        region_key][frame_num][2]
+            # We can only do a multi-peak fit after the single-peak
+            # Rayleigh fit is done, because we have to know the
+            # Rayleigh peak positions in GHz in order to convert
+            # the multi-peak fit bounds given in GHz into the position
+            # in pixels.
+            if nr_brillouin_peaks > 1:
+                ind =\
+                    (ind_x, ind_y, ind_z, slice(None), slice(None), 0)
+                rayleigh_peaks = np.transpose(
+                    evm.results['rayleigh_peak_position'][ind]
+                )
+                bounds = self.create_bounds(
+                    brillouin_regions_shifted,
+                    times,
+                    rayleigh_peaks
+                )
+                if bounds is not None:
+                    packed_data_multi_peak =\
+                        zip(irepeat(spectra),
+                            brillouin_regions_shifted,
+                            irepeat(nr_brillouin_peaks),
+                            bounds)
+                else:
+                    packed_data_multi_peak =\
+                        zip(irepeat(spectra),
+                            brillouin_regions_shifted,
+                            irepeat(nr_brillouin_peaks),
+                            irepeat(bounds))
+                # Process it
+                results_multi_peak = pool.starmap(
+                    self.fit_spectra, packed_data_multi_peak)
 
-                    if count is not None:
-                        count.value += 1
+                for frame_num, spectrum in enumerate(spectra):
+                    for region_key, _ in enumerate(
+                            brillouin_regions):
+                        ind = (ind_x, ind_y, ind_z,
+                               frame_num, region_key,
+                               slice(1, nr_brillouin_peaks+1))
+                        evm.results[
+                            'brillouin_peak_position'][ind] = \
+                            results_multi_peak[
+                                region_key][frame_num][0]
+                        evm.results[
+                            'brillouin_peak_fwhm'][ind] = \
+                            results_multi_peak[
+                                region_key][frame_num][1]
+                        evm.results[
+                            'brillouin_peak_intensity'][ind] = \
+                            results_multi_peak[
+                                region_key][frame_num][2]
 
-                    # Calculate the derived values every ten steps
-                    ind_total += 1
-                    if (ind_total % 10) == 0:
-                        calculate_derived_values()
+            # Calculate the shift of the Rayleigh peaks,
+            # in order to follow the peaks in case of a drift
+            rayleigh_peak_current = np.nanmean(
+                evm.results['rayleigh_peak_position'][
+                    ind_x, ind_y, ind_z, :, :, 0], axis=0)
+            # If we haven't found a valid Rayleigh peak position,
+            # but the current one is valid, use it
+            if not np.isnan(rayleigh_peak_current).all()\
+                    and np.isnan(rayleigh_peak_initial).all():
+                rayleigh_peak_initial = rayleigh_peak_current
+            shift = np.nanmean(rayleigh_peak_current - rayleigh_peak_initial)
+            if not np.isnan(shift):
+                rayleigh_shift = round(shift)
+
+            if count is not None:
+                count.value += 1
+
+            # Calculate the derived values every ten steps
+            if not (idx % 10):
+                calculate_derived_values()
 
         pool.close()
         pool.join()
@@ -677,36 +743,6 @@ class EvaluationController(object):
 
         return w0_bounds
 
-    def extract_payload_spectra(self, image_key):
-        em = self.session.extraction_model()
-        evm = self.session.evaluation_model()
-        if not em:
-            return None, None, None
-        time = self.session.get_payload_time(image_key)
-        arc = em.get_arc_by_time(time)
-        if arc.size == 0:
-            return None, None, None
-
-        imgs = self.session.get_payload_image(image_key)
-
-        # Extract values from *all* frames in the current payload
-        spectra = []
-        for img in imgs:
-            values_by_img = extract_lines_along_arc(
-                img,
-                arc
-            )
-            spectra.append(values_by_img)
-
-        exposure = self.session.current_repetition()\
-            .payload.get_exposure(image_key)
-        times = exposure * np.arange(len(imgs)) + time
-
-        intensities = np.nanmean(imgs, axis=(1, 2))
-
-        evm.set_spectra(image_key, spectra)
-        return spectra, times, intensities
-
     def get_data(self, parameter_key, brillouin_peak_index=0):
         """
         This function returns the evaluated data,
@@ -796,6 +832,33 @@ class EvaluationController(object):
 
         return data, positions, dimensionality, labels
 
+    @staticmethod
+    def get_key_from_indices(resolution, ind_x, ind_y, ind_z):
+        if len(resolution) != 3:
+            raise ValueError('resolution has wrong dimension')
+        if ind_x >= resolution[0]:
+            raise IndexError('x index out of range')
+        if ind_y >= resolution[1]:
+            raise IndexError('y index out of range')
+        if ind_z >= resolution[2]:
+            raise IndexError('z index out of range')
+        return str(ind_z * (resolution[0] * resolution[1])
+                   + ind_y * resolution[0] + ind_x)
+
+    @staticmethod
+    def get_indices_from_key(resolution, key):
+        key = int(key)
+        ind_z = floor(key / (resolution[0] * resolution[1]))
+        ind_y = floor(
+            (key - ind_z * (resolution[0] * resolution[1])) / resolution[0])
+        ind_x = (key % (resolution[0] * resolution[1])) % resolution[0]
+        # ind_y = (key - ind_x) % resolution[0]
+        if ind_x >= resolution[0]\
+                or ind_y >= resolution[1]\
+                or ind_z >= resolution[2]:
+            raise ValueError('Invalid key')
+        return ind_x, ind_y, ind_z
+
 
 def calculate_derived_values():
     """
@@ -983,3 +1046,39 @@ class Controller(object):
             evc.evaluate()
 
         return self.session
+
+
+class ExportController(object):
+
+    def __init__(self):
+        return
+
+    @staticmethod
+    def get_configuration():
+        return {
+            'fluorescence': {
+                'export': True,
+            },
+            'fluorescenceCombined': {
+                'export': True,
+            },
+            'brillouin': {
+                'export': True,
+                'shift': {
+                    'cax': ('min', 'max'),
+                }
+            },
+        }
+
+    def export(self, configuration=None):
+        if not configuration:
+            configuration = self.get_configuration()
+
+        FluorescenceExport().export(configuration)
+        FluorescenceCombinedExport().export(configuration)
+
+        # BrillouinExport needs the EvaluationController
+        # to nicely get the data, so we provide it here.
+        # Not really nice, but importing it in BrillouinExport
+        # leads to a circular dependency.
+        BrillouinExport(EvaluationController()).export(configuration)
