@@ -92,7 +92,20 @@ class Calibration(Serializer):
         self.num_brillouin_samples = num_brillouin_samples
         self.shift_methanol = shift_methanol
         self.shift_water = shift_water
+        self.shifts = np.array([])
+        self.orders = np.array([])
 
+        self.update_calibration()
+
+    def set_shift_water(self, shift_water):
+        self.shift_water = shift_water
+        self.update_calibration()
+
+    def set_shift_methanol(self, shift_methanol):
+        self.shift_methanol = shift_methanol
+        self.update_calibration()
+
+    def update_calibration(self):
         # Construct array with the frequency shifts
         tmp = [self.shift_methanol, self.shift_water]
         self.shifts = np.full(2 + 2 * self.num_brillouin_samples, 0.0)
@@ -101,7 +114,8 @@ class Calibration(Serializer):
             self.shifts[-1 * (i + 2)] = -1 * tmp[i]
 
         # The interference orders to which the peaks belong
-        self.orders = np.array([0, 0, 0, 1, 1, 1])
+        self.orders = np.full(2 + 2 * self.num_brillouin_samples, 0)
+        self.orders[-(1 + self.num_brillouin_samples):] = 1
 
 
 AVAILABLE_SETUPS = [
