@@ -5,6 +5,7 @@ from pathlib import Path
 
 import h5py
 from numpy import transpose
+import math
 
 from bmlab import __version__ as version
 from bmlab.file import BrillouinFile, is_source_file, is_session_file
@@ -167,6 +168,22 @@ class Session(Serializer):
             img = self.orientation.apply(imgs[0, ...])
             self.extraction_models.get(repetition).set_image_shape(img.shape)
 
+    def set_arc_width(self):
+        """
+        We set the arc_width for arc extraction on file load.
+        """
+        if self.file is None:
+            return
+
+        repetitions = self.file.repetition_keys()
+        for repetition in repetitions:
+            binning_factor = self.file.get_repetition(repetition)\
+                .payload.get_binning_factor('0')
+
+            em = self.extraction_models.get(repetition)
+            arc_width = math.ceil(em.arc_width / binning_factor)
+            em.set_arc_width(arc_width)
+
     @staticmethod
     def get_instance():
         """
@@ -223,9 +240,11 @@ class Session(Serializer):
                 for key in self.file.repetition_keys()
             }
             self.set_image_shape()
+            self.set_arc_width()
 
         try:
             self.load(file_name)
+            self.set_arc_width()
         except Exception as e:
             raise e
         else:
@@ -254,10 +273,27 @@ class Session(Serializer):
             return None
         return self.current_repetition().calibration.get_time(calib_key)
 
-    def get_image_keys(self):
+    def get_calibration_exposure(self, calib_key):
         if self.current_repetition() is None:
             return None
-        return self.current_repetition().payload.image_keys()
+        return self.current_repetition().calibration.get_exposure(calib_key)
+
+    def get_calibration_binning(self, calib_key):
+        if self.current_repetition() is None:
+            return None
+        return self.current_repetition().calibration.get_binning(calib_key)
+
+    def get_calibration_binning_factor(self, calib_key):
+        if self.current_repetition() is None:
+            return None
+        return self.current_repetition()\
+            .calibration.get_binning_factor(calib_key)
+
+    def get_image_keys(self, sort_by_time=False):
+        if self.current_repetition() is None:
+            return None
+        return self.current_repetition()\
+            .payload.image_keys(sort_by_time=sort_by_time)
 
     def get_payload_image(self, image_key, frame_num=None):
         if self.current_repetition() is None:
@@ -275,6 +311,22 @@ class Session(Serializer):
             return None
         return self.current_repetition().payload.get_time(image_key)
 
+    def get_payload_exposure(self, image_key):
+        if self.current_repetition() is None:
+            return None
+        return self.current_repetition().payload.get_exposure(image_key)
+
+    def get_payload_binning(self, image_key):
+        if self.current_repetition() is None:
+            return None
+        return self.current_repetition().payload.get_binning(image_key)
+
+    def get_payload_binning_factor(self, image_key):
+        if self.current_repetition() is None:
+            return None
+        return self.current_repetition()\
+            .payload.get_binning_factor(image_key)
+
     def get_payload_resolution(self):
         if self.current_repetition() is None:
             return None
