@@ -67,9 +67,13 @@ def test_file_get_repetition_keys():
     bf = BrillouinFile(data_file_path('Water.h5'))
     assert bf.repetition_keys() == ['0']
 
+    assert bf.repetition_keys('Fluorescence') == []
+
     bf = BrillouinFile(data_file_path('Water_old.h5'))
     assert bf.repetition_keys() == ['0']
 
+    assert bf.repetition_keys('Fluorescence') == []
+
 
 def test_file_get_repetition_date():
     bf = BrillouinFile(data_file_path('Water.h5'))
@@ -161,3 +165,81 @@ def test_file_calibration_get_time():
     bf = BrillouinFile(data_file_path('Water.h5'))
     time = bf.get_repetition('0').calibration.get_time('1')
     assert time == 36.802
+
+
+def test_file_calibration_get_exposure():
+    bf = BrillouinFile(data_file_path('Binning.h5'))
+    assert bf.get_repetition('0').calibration.get_exposure('1') == 0.8
+    assert bf.get_repetition('0').payload.get_exposure('0') == 0.2
+
+
+def test_file_calibration_get_binning():
+    bf = BrillouinFile(data_file_path('Binning.h5'))
+    assert bf.get_repetition('0').calibration.get_binning('1') == '8x8'
+    assert bf.get_repetition('0').payload.get_binning('0') == '8x8'
+
+    assert bf.get_repetition('0')\
+        .payload.get_binning_factor('0') == 8
+
+
+def test_file_repetition_count():
+    bf = BrillouinFile(data_file_path('Fluorescence.h5'))
+    mode = 'Fluorescence'
+
+    rep_keys = bf.repetition_keys(mode)
+    assert rep_keys == ['0', '1']
+
+    with pytest.raises(NotImplementedError):
+        bf.repetition_count('Raman')
+
+
+def test_file_get_fluorescence_images():
+    bf = BrillouinFile(data_file_path('Fluorescence.h5'))
+    mode = 'Fluorescence'
+
+    rep_keys = bf.repetition_keys(mode)
+    assert rep_keys == ['0', '1']
+
+    repetition = bf.get_repetition(rep_keys[0], mode)
+
+    image_keys = repetition.payload.image_keys()
+    assert image_keys == ['0', '1', '2', '3']
+
+    time = repetition.payload.get_time(image_keys[0])
+    assert time == 22.376
+
+    channel = repetition.payload.get_channel(image_keys[0])
+    assert channel == 'Blue'
+
+    roi = repetition.payload.get_ROI(image_keys[0])
+    assert roi == dict({
+        'bottom': 152,
+        'height_binned': 700,
+        'height_physical': 700,
+        'left': 200,
+        'right': 202,
+        'top': 150,
+        'width_binned': 600,
+        'width_physical': 600
+    })
+
+    image = repetition.payload.get_image(image_keys[0])
+    assert image.shape == (1, 700, 600)
+
+
+def test_file_get_scale_calibration():
+    bf = BrillouinFile(data_file_path('Fluorescence.h5'))
+    mode = 'Fluorescence'
+
+    repetition = bf.get_repetition('0', mode)
+    scale_calibration = repetition.payload.get_scale_calibration()
+
+    assert scale_calibration == dict({
+        'micrometerToPixX': (-7.95, 9.05),
+        'micrometerToPixY': (9.45, 8.65),
+        'pixToMicrometerX': (-0.05606325750210643, 0.05865577808023852),
+        'pixToMicrometerY': (0.0612482986583706, 0.05152634649037527),
+        'positionScanner': (0.0, 0.0),
+        'positionStage': (0.0, 0.0),
+        'origin': (0.0, 0.0)
+    })
